/*************************************************************************
 *   irisdong - show a rotating 3d dong mesh using IRIS GL
 *   Copyright (C) 2018 jrra
 * 
 *   to build:
 *       cc -o irisdong irisdong.c -lgl -lm
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <sys/time.h>
#include <sys/stat.h>

#include <gl/gl.h>
#include <math.h>
#include "mesh.h"

struct gfxhwdesctablenode {
	char *version;
	char *description;
	int end;
} hwtable[] = {
	/* This table associates the values returned from gversion()
	 * with a description of the graphics hardware.
	 * For more information, check the manpage of gversion. */
	{ "GL4D",	"B or G"				},
	{ "GL4DGT",	"GT"					},
	{ "GL4DGTX",	"GTX"					},
	{ "GL4DVGX",	"VGX"					},
	{ "GL4DVGXT",	"VGXT"					},
	{ "GL4DRE",	"RealityEngine"				},
	{ "GL4DPI",	"very early Personal Iris"		},
	{ "GL4DPI2",	"Personal Iris" 			},
	{ "GL4DPIT",	"Personal Iris with Turbo Graphics"	},
	{ "GL4DLG",	"Entry"					},
	{ "GL4DXG",	"XS, XS24, XZ, Elan, or Extreme"	},
	{ "GL4DNP",	"Indy"					},
	{ "GL4DSLD",	"Solid IMPACT"				},
	{ "GL4DIMP",	"High IMPACT or Maximum IMPACT"		},
	{ "GL4DINF",	"InfiniteReality"			},
	{ "GL4DIOO",	"VPro"					},
	{ "", "", 1 },	/* end-of-array sentinel */
};

const int winWidth  = 640;
const int winHeight = 480;

int frame = 0;
int frametime = 0;
struct timeval tvPrev, tvCur, tvDiff;
int rootx, rooty, winx, winy;
unsigned int mask_return;
int inWindow = 0;
float rotation = 0.0;
int wireframe = 0;

static float mat[] = {
	AMBIENT, .1, .1, .1,
	DIFFUSE, 0, .369, .165,
	SPECULAR, .5, .5, .5,
	SHININESS, 30,
	LMNULL
};

static float lt[] = {
	LCOLOR, 1, .8, .8,
	/*POSITION, 0, 1.5, -.5, 1,*/
	POSITION, 0, 1000, 1000, 1,
	LMNULL
};

static float lm[] = {
	AMBIENT, .1, .1, .1,
	LOCALVIEWER, 1,
	LMNULL
};


void die( char *msg )
{
    fprintf( stderr, "%s\n", msg );
    exit(1);
}

#ifndef timersub
/* needed for IRIX.... */
void timersub( struct timeval *a, struct timeval *b, struct timeval *res ) {
    res->tv_sec = a->tv_sec - b->tv_sec;
    res->tv_usec = a->tv_usec - b->tv_usec;
    if( res->tv_usec < 0 )
    {
        res->tv_sec --;
        res->tv_usec += 1000000;
    }
}
#endif

char *getDescFromVersion(char *v)
{
	char version[13];
	char *dash;
	int i;
	
	strncpy(version, v, 12);
	dash = strchr(version, '-');
	if (dash)
		*dash = '\0';
	else
		return "unknown";
	
	i=0;
	while(!hwtable[i].end) {
		if( !strcmp( version, hwtable[i].version ) )
			return hwtable[i].description;
		++i;
	}
	return "unknown";
}

int main( int argc, char *argv[] ) {
	char version[13];
	float rescaledNormals[NUM_VERTS][3];
	int i,j;
	int min,max;
	float rotation=0.0;
	Object model;
	printf("IrisGL implementation is %s.\n", getgdesc(GD_IS_NATIVE) ?
		"native" :
		"non-native, layered on OpenGL"
	);
	printf("multisampling %s.\n", getgdesc(GD_MULTISAMPLE) ?
		"available" :
		"not available"
	);
	printf("screen type is %s.\n", getgdesc(GD_SCRNTYPE)==GD_SCRNTYPE_WM ?
		"GD_SCRNTYPE_WM (has window manager)" :
		"GD_SCRNTYPE_NOWM (no window manager)"
	);
	printf("GD_WSYS is %s.\n", getgdesc(GD_WSYS)==GD_WSYS_4S ?
		"GD_WSYS_4S (windowing system currently running)" :
		"GD_WSYS_NONE (no window system currently running)"
	);
	gversion(version);
	printf("gversion is %s (%s).\n", version, getDescFromVersion(version));
	
	/* calculate normalized normals */
	for(i=0; i<NUM_VERTS; i++) {
		double magnitude = 0.0;
		double nx = normals[i][0];
		double ny = normals[i][1];
		double nz = normals[i][2];
		magnitude = sqrt(nx*nx + ny*ny + nz*nz);
		nx /= magnitude;
		ny /= magnitude;
		nz /= magnitude;
		rescaledNormals[i][0] = nx;
		rescaledNormals[i][1] = ny;
		rescaledNormals[i][2] = nz;
	}
	
	prefsize(640, 480);
	winopen("irisdong");
	doublebuffer();
	RGBmode();
	gconfig();
	mmode(MVIEWING);
	perspective(900, 4.0/3.0, 0, 500);
	lookat(0,0,1000,0,0,0,0);
	
	cpack(0);
	clear();
	swapbuffers();
	clear();
	
	/* generate model object */
	makeobj(model=genobj());
	for(i=0;i<NUM_TRIS;i++) {
		bgnpolygon();
		for(j=0;j<3;j++) {
			n3f(rescaledNormals[tris[i][j]]);
			v3s(verts[tris[i][j]]);
		}
		endpolygon();
	}
	closeobj();
	
	lmdef(DEFMATERIAL, 39, 0, mat);
	lmdef(DEFLIGHT, 27, 0, lt);
	lmdef(DEFLMODEL, 14, 0, lm);
	lmbind(MATERIAL, 39);
	lmbind(LIGHT0, 27);
	lmbind(LMODEL, 14);
	
	zbuffer(TRUE);
 	backface(TRUE);
	polymode(PYM_FILL);
	while(1) {
		pushmatrix();
		zfunction(ZF_GREATER);
		czclear(0, 0x800000);
		cpack(0xff626262);
		if(rotation >= 3600.0)
			rotation -= 3600.0;
		rotation += 2;
		rotate(rotation, 'y');
		callobj(model);
		swapbuffers();
		popmatrix();
	}
	gexit();
	
	return 0;
}
